#!/usr/bin/python3
#
#       contRun_altnSource_chtChtss.py
#
#       OpenFOAMが出力した熱流束から、FrontISTRを実行して、
#       温度、熱ひずみを出力する。
#       contRun_altnSource_chtChtss.pyを交互計算用に修正。
#
#   23/10/23    saveMappingTemp:交互計算に対応させる為、
#               tempのfile名のstep数をendStepからtimeIndexに変更。
#               addTempRenameRes,setDeltaTInCnt_cht,deleteResSetRestartFileCht:
#               大幅修正。
#               createCommTimeArea:新規追加。（前回のcouplingTime取得の為）
#               runFistr:Fistr起動前に、Fistrが停止している事を確認後、Fistrを起動
#               runCoupling.pyから起動された場合、直ぐに戻る用に修正。
#

import sys, os
import mmap
import pickle
import time
import shutil
import multiprocessing
import pyCoupling
import pyFistr_ptc as pyFistr
import mappingFromPointCloud as mapping


waitCount = 5000
waitTime = 0.01


#--------------------
#  createCommonArea
#--------------------
def createCommonArea():
    """ common fileを共有メモリとして設定する。
    200文字で設定。"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = "common"
    commonFile = dataDir + "/" + name
    f = open(commonFile, "r+b"); mmFile = mmap.mmap(f.fileno(), 0); f.close()
    return mmFile

#
#  writeCommonArea
def writeCommonArea(cont):
    """ 共有メモリ（200文字）への書き込み。"""
    global mmCommon
    length = 200
    space = "\00" * (length - len(cont))
    cont += space
    mmCommon.seek(0)
    mmCommon.write(cont.encode())
    mmCommon.flush()

#
#  readCommonArea
def readCommonArea():
    """ 共有メモリの内容を読み込む"""
    global mmCommon
    mmCommon.seek(0)
    cont = mmCommon.read().decode()
    n = cont.find("\00")
    cont = cont[:n]
    return cont

#--------------------------
#  createCommTimePressArea
#--------------------------
def createCommTimePressArea():
    """ 共有メモリ作成
    couplingTime, stepのセットを最新から3組保存する領域"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = "commTime"
    commonFile = dataDir + "/" + name
    f = open(commonFile, "r+b"); mmFile = mmap.mmap(f.fileno(), 0); f.close()
    return mmFile

#
#  writeCommTimePressArea
def writeCommTimePressArea(cont):
    """ 共有メモリ（200文字）への書き込み。"""
    global mmCommTimePress
    length = 200
    space = "\00" * (length - len(cont))
    cont += space
    mmCommTimePress.seek(0)
    mmCommTimePress.write(cont.encode())
    mmCommTimePress.flush()

#
#  readCommTimePressArea
def readCommTimePressArea():
    """ 共有メモリの内容を読み込む"""
    global mmCommTimePress
    mmCommTimePress.seek(0)
    cont = mmCommTimePress.read().decode()
    n = cont.find("\00")
    cont = cont[:n]
    return cont

#
#  getBeforeCouplingTimePress
def getBeforeCouplingTimePress():
    """ 直前のcouplingTime、stepを共有メモリから取得"""
    cont = readCommTimePressArea()
    lines = cont.split("\n")[:-1]
    if len(lines) <= 1:
        cpTime = pyCoupling.time2str(-1)
        cpStep = "-1"
        return cpStep, cpTime
    line = lines[-2]
    cpTime, cpStep = line.split()
    return cpStep, cpTime

#--------------------------
#  createCommTimeTempArea
#--------------------------
def createCommTimeTempArea():
    """ 共有メモリ作成
    couplingTime, stepのセットを最新から3組保存する領域"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = "commTimeTemp"
    commonFile = dataDir + "/" + name
    f = open(commonFile, "r+b"); mmFile = mmap.mmap(f.fileno(), 0); f.close()
    return mmFile

#
#  writeCommTimeTempArea
def writeCommTimeTempArea(cont):
    """ 共有メモリ（200文字）への書き込み。"""
    global mmCommTimeTemp
    length = 200
    space = "\00" * (length - len(cont))
    cont += space
    mmCommTimeTemp.seek(0)
    mmCommTimeTemp.write(cont.encode())
    mmCommTimeTemp.flush()

#
#  readCommTimeTempArea
def readCommTimeTempArea():
    """ 共有メモリの内容を読み込む"""
    global mmCommTimeTemp
    mmCommTimeTemp.seek(0)
    cont = mmCommTimeTemp.read().decode()
    n = cont.find("\00")
    cont = cont[:n]
    return cont

#
#  getBeforeCouplingTimeTemp
def getBeforeCouplingTimeTemp():
    """ 直前のcouplingTime、stepを共有メモリから取得"""
    cont = readCommTimeTempArea()
    lines = cont.split("\n")[:-1]
    if len(lines) <= 1:
        cpTime = pyCoupling.time2str(-1)
        cpStep = "-1"
        return cpStep, cpTime
    line = lines[-2]
    cpTime, cpStep = line.split()
    return cpStep, cpTime

#-------------------------
#  saveCouplingTimePress
#-------------------------
def saveCouplingTimePress():
    """ couplintTime, stepのペアを保存"""
    timeName = couplingDict["timeName"]
    timeIndex = couplingDict["timeIndex"]
    procNo = couplingDict["procNo"]
    if int(procNo) == 0:
        addLine = timeName + " " + timeIndex
        cont = readCommTimePressArea()
        cont += addLine + "\n"
        lines = cont.split("\n")[:-1]
        lines = lines[-3:]
        cont = "\n".join(lines) + "\n"
        writeCommTimePressArea(cont)

#-------------------------
#  saveCouplingTimeTemp
#-------------------------
def saveCouplingTimeTemp():
    """ couplintTime, stepのペアを保存"""
    timeName = couplingDict["timeName"]
    timeIndex = couplingDict["timeIndex"]
    procNo = couplingDict["procNo"]
    if int(procNo) == 0:
        addLine = timeName + " " + timeIndex
        cont = readCommTimeTempArea()
        cont += addLine + "\n"
        lines = cont.split("\n")[:-1]
        lines = lines[-3:]
        cont = "\n".join(lines) + "\n"
        writeCommTimeTempArea(cont)

#---------------------
#  coupling_chtChtss
#---------------------
def coupling_chtChtss():
    """ OpenFOAMが出力した熱流束から、FrontISTRが温度を計算する。"""
    solidCaseDir = couplingDict["solidCaseDir"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    patchName = couplingDict["mappingPatch"]
    timeName = couplingDict["timeName"]
    procNo = couplingDict["procNo"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    #---- heatFluxをmapping ---------
    #heatFluxの点群ファイル作成
    htFile = dataDir + "/" + patchName + "_faceHeatFlux_" + timeName + "_" + procNo + "tm"
    createPtcFileHeatFlux(htFile)
    if int(procNo) == 0:
        #heatFluxをmappingしてcntファイルに反映
        createMappingHeatFlux()
        #---- fistr実行（温度計算） --------------
        #cht計算前の設定
        endStep, endTime = setFrontIstr_cht()
        #Fistr起動
        runFistr("cht", endStep, endTime)
        #mshFileを更新、writeInterval時のrename
        tempLines = addTempRenameRes(endStep, endTime)
        #--- 温度を取得 ---------
        #温度のptcデータを作成
        print("(py) --- creating temperature PTC ...", flush=True)
        tempPtc = createTempPtcFile(tempLines)
        #温度をmappingしたfileを作成
        print("(OF) --- temperature value is mapping to patch '" + patchName + "' ...", flush=True)
        createMappingTempFile(tempPtc, endStep)
        #---- fistr実行（熱ひずみ計算）----------
        #chtss計算前の設定(cnt,mshファイルを設定)
        setFsiChtCntFile("chtss")
        #温度の計算結果をcntファイルに設定
        remakeFistrCntFileForTemperature(tempLines)
        #Fistr起動(endStep, endTimeはdummy)
        runFistr("chtss", endStep, endTime)
        #---- 変位を取得 -----------------------
        #変位の点群データ作成
        print("(py) --- creating displacement PTC ...", flush=True)
        chtssDispPtc = createChtssDispPtc()
        #変位のmappingFileを作成
        print("(py) --- displacement value is mapping to patch '" + patchName + "' ...", flush=True)
        createMappingDispFile(chtssDispPtc, endStep)
        #--- 結果file整理（file削除、restart設定）----
        deleteResSetRestartFileCht()
    return ""

#
#  createPtcFileHeatFlux
#------------------------
def createPtcFileHeatFlux(htFile):
    """ 点群file作成"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    solidCaseDir = couplingDict["solidCaseDir"]
    timeName = couplingDict["timeName"]
    patchName = couplingDict["mappingPatch"]
    procNo = couplingDict["procNo"]
    #faceCenterLoc読み込み
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    faceCenterFile = dataDir + "/" + patchName + "_faceCenterLocs_" + procNo
    f = open(faceCenterFile); lines = f.readlines(); f.close()
    #faceValueを読み込み
    fp = open(htFile); pLines = fp.readlines(); fp.close()
    newLines = []
    for i in range(len(lines)):
        newLine = lines[i][:-1] + " " + pLines[i]
        newLines.append(newLine)
    #heatFluxのptcファイルを保存
    name = patchName + "_faceCenterHeatFlux_" + timeName + "_" + procNo + ".particles"
    ptcFileName = solidCaseDir + "/" + name
    f = open(ptcFileName, "w"); f.writelines(newLines); f.close()
    return

#
#  createMappingHeatFlux
#------------------------
def createMappingHeatFlux():
    """ 各procNo毎のheatFlux点群fileをFistr側にmappingする"""
    procNo = couplingDict["procNo"]
    nProcs = couplingDict["nProcs"]
    patchName = couplingDict["mappingPatch"]
    timeName = couplingDict["timeName"]
    sgrpName = couplingDict["mappingSgrp"]
    solidCaseDir = couplingDict["solidCaseDir"]
    sgrpDictName = solidCaseDir + "/sgrpDict.bin"
    f = open(sgrpDictName, "rb")
    sgrpDict = pickle.load(f)
    f.close()
    nodeDictName = solidCaseDir + "/nodeDict.bin"
    f = open(nodeDictName, "rb")
    nodeDict = pickle.load(f)
    f.close()
    #各procno毎の点群fileをまとめる
    lines = []
    if int(procNo) == 0:
        for n in range(int(nProcs)):
            pNo = str(n)
            name = patchName + "_faceCenterHeatFlux_" + timeName + "_" + pNo + ".particles"
            eachPtcFile = solidCaseDir + "/" + name
            waitUntilExistingFile(eachPtcFile)
            f = open(eachPtcFile)
            lines += f.readlines()
            f.close()
        newName = patchName + "_faceCenterHeatFlux.particles"
        allPtcFile = solidCaseDir + "/" + newName
        f = open(allPtcFile, "w"); f.writelines(lines); f.close()
        #ptcData(数値data）を取得
        mesh = mapping.fistrMesh(solidCaseDir)
        ptcData = mesh.getPtcDataFromLines(lines)
        #mapping用の3点を取得
        mapStat, mappingTriDict = mesh.getMappingTriLocs(ptcData, sgrpDict, nodeDict)
        #heatFluxをmapping
        print("(py) --- heat flux value is mapping to SGRP '" + sgrpName + "' of FrontISTR...", flush=True)
        mappingDict = mesh.setMappingDataFromTriLocs(ptcData, sgrpDict, mappingTriDict)
        #mapping結果を保存
        lines = mesh.createMappingFile(mappingDict, sgrpName, allPtcFile)
        name = "mappingSHF_" + sgrpName + ".map"
        fileName = solidCaseDir + os.sep + name
        f = open(fileName, "w"); f.writelines(lines); f.close()
        #mapping結果をcntファイルに反映
        remakeFistrCntFileForHeatFlux(lines)

#
#  remakeFistrCntFileForHeatFlux
def remakeFistrCntFileForHeatFlux(mapLines):
    """ heatFluxのmapping結果をcntファイルに反映"""
    solidCaseDir = couplingDict["solidCaseDir"]
    sgrpName = couplingDict["mappingSgrp"]
    addLines = []
    for line in mapLines:
        if line[0] != "#":
            words = line.split()
            if len(words) >= 3:
                words[1] = "S" + words[1]
                newLine = ", ".join(words) + "\n"
                addLines.append(newLine)
    #cntFileを修正
    cntFile = solidCaseDir + os.sep + "FistrModel_cht.cnt"
    f = open(cntFile); lines = f.readlines(); f.close()
    #挿入場所は、最初の!DFLUXの該当SGRPに挿入する。
    i = 0
    while i < len(lines):
        i = pyFistr.getNextHeaderName(lines, i, "!DFLUX", "")
        i += 1
        if i >= len(lines):
            mess = "error: '" + sgrpName + "' does not exist in !DFLUX.\n"
            print(mess)
            si = i
            break
        words = pyFistr.deleteSp(lines[i]).split(",")
        if words[0] == sgrpName:
            si = i - 1
            ei = pyFistr.skipNextHeader(lines, i)
            break
    if si < len(lines):
        #cntFile修正、書き込み
        name = "mappingSHF_" + sgrpName + ".map"
        grpid = pyFistr.getValName(lines[si], "GRPID")
        words = ["!DFLUX", "GRPID="+grpid, "forceType=mapping", "mapFile="+name,
                 "mapCoeff=1.0"]
        lines[si] = ", ".join(words) + "\n"
        lines[si+1] = sgrpName + ", S1, 0.0\n"
        newLines = lines[:si+2] + addLines + lines[ei:]
        f = open(cntFile, "w"); f.writelines(newLines); f.close()

#
#  setFrontIstr_cht
#----------------------
def setFrontIstr_cht():
    """ FrontISTRのcht計算前の設定"""
    #msh,cntファイル名を設定
    setFsiChtCntFile("cht")
    #時間増分を更新
    endStep, endTime = setDeltaTInCnt_cht()
    return endStep, endTime

#
#  setFsiChtCntFile
def setFsiChtCntFile(sect):
    """ sectに応じたdatファイルに修正"""
    solidCaseDir = couplingDict["solidCaseDir"]
    pasteDatFile = solidCaseDir + os.sep + "hecmw_ctrl.dat"
    if sect == "fsi":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_fsi.dat"
    elif sect == "cht":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_cht.dat"
    elif sect == "chtss":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_chtss.dat"
    shutil.copy(datFile, pasteDatFile)

#
#  setDeltaTInCnt_cht
def setDeltaTInCnt_cht():
    """ chtのcntファイルを更新する。"""
    currTime = couplingDict["timeName"]         #endTime
    deltaT = couplingDict["deltaT"]
    timeIndex = couplingDict["timeIndex"]       #endStep
    solidCaseDir = couplingDict["solidCaseDir"]
    nSteps = couplingDict["nSteps"]
    endStep = int(timeIndex)
    #startStep = int(timeIndex) - int(nSteps)
    eTime = float(currTime)
    #sTime = getCouplingTimeTempFromStep(startStep)
    sStep, sTime = getBeforeCouplingTimeTemp()
    startStep = int(sStep)
    stepTime = (eTime - float(sTime)) / (int(timeIndex) - int(sStep))
    if stepTime == 0.0:
        stepTime = float(deltaT)
    endTime = eTime
    #cnt読み込み
    cntFile = solidCaseDir + "/FistrModel_cht.cnt"
    f = open(cntFile); lines = f.readlines(); f.close()
    #時間増分、終了stepを修正
    i = pyFistr.getNextHeaderName(lines, 0, "!HEAT", "")
    i += 1
    words = pyFistr.deleteSp(lines[i]).split(",")
    words[0] = str(stepTime)            #時間増分
    dT = stepTime
    endTimeStep = endStep - startStep
    edTime = endTimeStep * dT
    words[1] = str(edTime)
    lines[i] = ", ".join(words) + "\n"
    #cntFile保存
    f = open(cntFile, "w"); f.writelines(lines); f.close()
    return endStep, endTime

#
#  remakeFistrCntFileForTemperature
#------------------------------------
def remakeFistrCntFileForTemperature(tempLines):
    """ temperatureの計算結果をcntファイルに反映"""
    solidCaseDir = couplingDict["solidCaseDir"]
    #cntFileを修正
    cntFile = solidCaseDir + os.sep + "FistrModel_chtss.cnt"
    f = open(cntFile); lines = f.readlines(); f.close()
    #挿入場所は、!TEMPERATUREのALLに、'otherData=yes'を追加して挿入。
    si = 0
    i = 0
    while i < len(lines):
        i = pyFistr.getNextHeaderName(lines, i, "!TEMPERATURE", "")
        i += 1
        if i >= len(lines):
            break
        words = pyFistr.deleteSp(lines[i]).split(",")
        if words[0] == "ALL":
            #!TEMPERATUREの行に'otherData=yes'を追加
            lines[i-1] = pyFistr.setValName(lines[i-1], "otherData=yes")
            si = i + 1
            ei = pyFistr.skipNextHeader(lines, i)
            break
    if si == 0:
        print("error: could not find !TEMPERATURE card!")
        return
    #tempLinesを追加
    lines = lines[:si] + tempLines + lines[ei:]
    f = open(cntFile, "w"); f.writelines(lines); f.close()

#
#  runFistr
#----------
def runFistr(sect, endStep, endTime):
    """ FrontISTRを起動"""
    timeName = couplingDict["timeName"]
    timeIndex = couplingDict["timeIndex"]
    nSteps = couplingDict["nSteps"]
    startStep = endStep - int(nSteps)
    #Fistrがfreeまで待つ
    waitUntilFreeFistr()
    if sect == "cht":
        print("(py) --- FrontISTR is running for heat transfer from step "
            #+ timeIndex + " to " + str(endStep) + " time "
            + str(startStep) + " to " + str(endStep) + " time "
            + str(endTime) + " ...", flush=True)
        #起動設定
        stepCont = sect + " -> " + str(endStep) + " steps"
        title = "title runFistr\n"
        time = "time " + timeName + "\n"
        footer = "footer " + "FrontISTR total time (" + stepCont + ") ----> \n"
        cont = title + time + footer + "#end\n"
    elif sect == "chtss":
        print("(py) --- FrontISTR is running for heat stress...", flush=True)
        #起動設定
        stepCont = sect + " -> " + str(endStep) + " steps"
        title = "title runFistr\n"
        time = "time " + timeName + "\n"
        footer = "footer " + "FrontISTR totla time (" + sect + ") ----> \n"
        cont = title + time + footer + "#end\n"
    writeCommonArea(cont)
    #Fistr終了まで待つ
    waitUntilFreeFistr()

#
#  waitUntilFreeFistr
def waitUntilFreeFistr():
    """ Fistrの終了を待つ。
    共有メモリの内容から確認する"""
    count = waitCount
    sleepTime = waitTime
    while True:
        cont = readCommonArea()
        if len(cont) > 0:
            lines = cont.split("\n")
            if len(lines) > 0:
                words = lines[0].split()
                if len(words) >= 2:
                    if words[1] == "FistrFinished":
                        break
        else:
            break
        count -= 1
        if count < 0:
            break
        time.sleep(sleepTime)
    return

#
#  addTempRenameRes
#--------------------
def addTempRenameRes(endStep, endTime):
    """ mshFileに温度の計算結果を追加してrenameする"""
    timeName = couplingDict["timeName"]
    timeIndex = couplingDict["timeIndex"]
    #deltaT = couplingDict["deltaT"]
    writeInterval = couplingDict["writeInterval"]
    nSteps = couplingDict["nSteps"]
    solidCaseDir = couplingDict["solidCaseDir"]
    #startStep = int(timeIndex) - int(nSteps)
    #startTime = getCouplingTimeTempFromStep(startStep)
    ssStep, ssTime = getBeforeCouplingTimeTemp()
    startTime = float(ssTime)
    stepTime = (endTime - startTime) / int(nSteps)
    if stepTime <= 0:
        stepTime = float(couplingDict["deltaT"])
    deltaT = str(stepTime)
    #温度データを取得
    #edTime = endTime - float(startTime)
    #nSteps = int(edTime / float(couplingDict["deltaT"]) + 0.5)
    #nSteps = int(couplingDict["nSteps"])
    nSteps = int(timeIndex) - int(ssStep)
    resFile = solidCaseDir + os.sep + "FistrModel_temp.res.0." + str(nSteps)
    tempLines = getTempLinesInResFile(resFile)
    nextTime = endTime
    if pyCoupling.isWriteInterval(nextTime, writeInterval, deltaT) == True:
        if nSteps != endStep:
            #温度データをrename
            newName = solidCaseDir + os.sep + "FistrModel_temp.res.0." + str(endStep)
            os.rename(resFile, newName)
            #vtuFolder、vtuFileをrename
            nn = "%04d" % nSteps
            nnEnd = "%04d" % endStep
            orgName = "FistrModel_cht.vis_psf." + nn
            newName = "FistrModel_cht.vis_psf." + nnEnd
            vtuFolder = solidCaseDir + "/" + orgName
            newFolder = solidCaseDir + "/" + newName
            if os.path.exists(newFolder) == True:
                shutil.rmtree(newFolder)
            shutil.move(vtuFolder, newFolder)
            vtuFile = newFolder + "/" + orgName + ".0.vtu"
            newFile = newFolder + "/" + newName + ".0.vtu"
            shutil.move(vtuFile, newFile)
            #pvtuFileをrename
            pvtuFile = solidCaseDir + "/" + orgName + ".pvtu"
            newFile = solidCaseDir + "/" + newName + ".pvtu"
            if os.path.exists(newFile) == True:
                os.remove(newFile)
            shutil.move(pvtuFile, newFile)
            #  1行修正
            replaceLineInPvtuFile(newFile, orgName, newName)
    #meshFileに温度データを初期値として追加
    addTempDataToMeshFile(tempLines)
    return tempLines

#
#  replaceLineInPvtuFile
def replaceLineInPvtuFile(newFile, orgName, newName):
    """ pvtuファイル内の「Source=」行を修正"""
    f = open(newFile); lines = f.readlines(); f.close()
    for i in range(len(lines)):
        line = lines[i]
        n = line.find("Source=")
        if n >= 0:
            newLine = '<Piece Source="./' + newName + '/' + newName + '.0.vtu"/>\n'
            lines[i] = newLine
            break
    f = open(newFile, "w"); f.writelines(lines); f.close()

#
#  getTempLinesInResFile
def getTempLinesInResFile(resFile):
    """ resFileから温度データを取得する"""
    tempLines = []
    f = open(resFile); lines = f.readlines(); f.close()
    flag = 0
    i = 0
    while i < len(lines):
        if flag == 1:
            nodeNo = lines[i].split()[0]
            temp = lines[i+1].split()[0]
            tempLines.append(", ".join([nodeNo, temp]) + "\n")
            i += 1
        elif lines[i][:len("TEMPERATURE")] == "TEMPERATURE":
            flag = 1
        i += 1
    return tempLines

#
#  addTempDataToMeshFile
def addTempDataToMeshFile(tempLines):
    """ mshFileに温度データを追加"""
    solidCaseDir = couplingDict["solidCaseDir"]
    mshFile = solidCaseDir + os.sep + "FistrModel_cht.msh"
    f = open(mshFile); lines = f.readlines(); f.close()
    headerData = pyFistr.getHeaderData(lines)
    for i in range(len(headerData)):
        header = headerData[i][0]
        if header[:len("!INITIAL CONDITION")] == "!INITIAL CONDITION":
            headerData[i][1] = tempLines
            break
    lines = []
    for header in headerData:
        lines += [header[0]]
        lines += header[1]
    f = open(mshFile, "w"); f.writelines(lines); f.close()

#
#  createTempPtcFile
#----------------------
def createTempPtcFile(tempLines):
    """ 温度の点群fileを作成"""
    #FrontISTRのnodeDictを取得
    solidCaseDir = couplingDict["solidCaseDir"]
    nodeDictName = solidCaseDir + "/nodeDict.bin"
    f = open(nodeDictName, "rb")
    nodeDict = pickle.load(f)
    f.close()
    #温度の点群データを取得作成
    locTemp = createTempPtc(nodeDict, tempLines)
    return locTemp

#
#  createTempPtc
def createTempPtc(nodeDict, tempLines):
    """ 温度の点群データ取得"""
    tempDict = {}
    for line in tempLines:
        words = pyFistr.deleteSp(line).split(",")
        nodeNo = int(words[0])
        temp = float(words[1])
        tempDict[nodeNo] = temp
    tempPtc = []
    for nodeNo in nodeDict.keys():
        temp = tempDict[nodeNo]
        loc = nodeDict[nodeNo]
        tempPtc.append([loc, temp])
    return tempPtc

#
#  createMappingTempFile
#------------------------
def createMappingTempFile(tempPtc, endStep):
    """ 温度をpatchにmappingしたfileを作成する"""
    nProcs = couplingDict["nProcs"]
    np = int(nProcs)
    queue = []
    procs = []
    for n in range(np):
        q = multiprocessing.Queue()
        queue.append(q)
        procNo = str(n)
        p = multiprocessing.Process(
            target=createMappingTempFile_proc,
            args=(tempPtc, endStep, procNo, q)
            )
        p.start()
        procs.append(p)
    #全セットprocessの終了を待つ
    for n in range(np):
        a = queue[n].get()

#
#  createMappingTempFile_proc
def createMappingTempFile_proc(tempPtc, endStep, procNo, q):
    """ 指定したprocNoの変位をpatchにmappingしたfileを作成する"""
    solidCaseDir = couplingDict["solidCaseDir"]
    mesh = mapping.fistrMesh(solidCaseDir) 
    #mappingするpointの座標を取得(face中心座標、point座標の辞書)
    mapFaceCenterDict, mapPointDict = getMappedPointLocsCht(procNo)
    #mappingTriFileがあるか？
    if isMappingTriFileCht(procNo) == False:
        #存在しない場合、mappingTriDictを取得
        mapStat, mappingTriDict = mesh.getMappingTriLocs(tempPtc, mapFaceCenterDict, mapPointDict)
        if mapStat != "OK":
            return "error in displacement mapping."
        #mappingTriFileを作成保存
        createMappingTriFileCht(mappingTriDict, procNo)
    else:
        #fileを読み込む
        mappingTriDict = readMappingTriFileCht(procNo)
    #変位をmapping
    mappingDict = mesh.setMappingDataFromTriLocs(tempPtc, mapFaceCenterDict, mappingTriDict)
    #mapping結果を保存(このfileをOpenFOAM側が読み込む)
    saveMappingTemp(mappingDict, endStep, procNo)
    q.put("")

#
#  getMappedPointLocsCht
def getMappedPointLocsCht(procNo):
    """ 温度をmappingするpointの座標（face中心座標）を取得する"""
    patchName = couplingDict["mappingPatch"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    #mapPointDictを取得
    name = patchName + "_mapPointFaceNodes_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    mapPointDict = {}
    for line in lines:
        words = line.split()
        if len(words) > 4:
            nodeNo = int(words[0])
            if not (nodeNo in mapPointDict.keys()):
                pointLoc = list(map(float, words[1:4]))
                mapPointDict[nodeNo] = pointLoc
    #mapFaceCenterDictを作成
    name = patchName + "_mapFaceCenterNodes_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    mapFaceCenterDict = {}
    for line in lines:
        words = line.split()
        if len(words) >= 7:
            faceNo = int(words[0])
            faceLoc = list(map(float, words[1:4]))
            nodeNos = list(map(int, words[4:]))
            mapFaceCenterDict[faceNo] = [faceLoc, nodeNos]
    return mapFaceCenterDict, mapPointDict

#
#  isMappingTriFileCht
def isMappingTriFileCht(procNo):
    """ 温度mapping用の3点を取得したfileが存在するかチェック"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    name = "mappingTriFileCht_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    return os.path.exists(fileName)

#
#  createMappingTriFileCht
def createMappingTriFileCht(mappingTriDict, procNo):
    """ mapping用の3点の座標をfileに保存する"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    name = "mappingTriFileCht_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    lines = []
    for key in mappingTriDict.keys():
        faceNo = key
        word = str(faceNo)
        triLocs = mappingTriDict[key]
        loc0, loc1, loc2 = triLocs
        word0 = " ".join(list(map(str, loc0)))
        word1 = " ".join(list(map(str, loc1)))
        word2 = " ".join(list(map(str, loc2)))
        line = " ".join([word, word0, word1, word2]) + "\n"
        lines.append(line)
    f = open(fileName, "w"); f.writelines(lines); f.close()

#
#  readMappingTriFileCht
def readMappingTriFileCht(procNo):
    """ 保存したmapping用の3点の座標を読み込む"""
    #procNo = couplingDict["procNo"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    name = "mappingTriFileCht_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    mappingTriDict = {}
    for line in lines:
        words = line.split()
        faceNo = int(words[0])
        key = faceNo
        loc0 = list(map(float, words[1:4]))
        loc1 = list(map(float, words[4:7]))
        loc2 = list(map(float, words[7:10]))
        mappingTriDict[key] = [loc0, loc1, loc2]
    return mappingTriDict

#
#  saveMappingTemp
def saveMappingTemp(mappingDict, endStep, procNo):
    """ mappingした温度をfileに保存。このファイルをOF側が読み込む"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    patchName = couplingDict["mappingPatch"]
    timeIndex = couplingDict["timeIndex"]
    saveDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = patchName + "_mapFaceCenterNodes_" + procNo
    fileName = saveDir + "/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    #保存のfilename
    name = patchName + "_temp_" + timeIndex + "_" + procNo + "tm"
    if procNo == "0":
        #print("(py) --- temperature file is creating for OpenFOAM at " + str(endStep) + "...")
        print("(py) --- temperature file is creating for OpenFOAM at " + name + "...")
    #name = patchName + "_temp_" + str(endStep) + "_" + procNo + "tm"
    tempFile = saveDir + "/" + name
    f = open(tempFile, "w")
    for line in lines:
        words = line.split()
        nodeNo = int(words[0])
        temp = mappingDict[nodeNo]
        line = str(temp) + "\n"
        f.write(line)
    f.close()
    return

#
#  createChtssDispPtc
#---------------------
def createChtssDispPtc():
    """ chtss変位の点群データ取得"""
    solidCaseDir = couplingDict["solidCaseDir"]
    nodeDictName = solidCaseDir + "/nodeDict.bin"
    f = open(nodeDictName, "rb")
    nodeDict = pickle.load(f)
    f.close()
    #resFile名を取得、読み込み
    resFile = solidCaseDir + "/FistrModel.res.0.1"
    dispDict = readDispFromResFile(resFile)
    #点群作成
    dispPtc = []
    for nodeNo in nodeDict.keys():
        disp = dispDict[nodeNo]
        loc = nodeDict[nodeNo]
        dispPtc.append([loc, disp])
    return dispPtc

#
#  readDispFromResFile
def readDispFromResFile(resFile):
    """ 変位データをresFileから取得"""
    #resFileを読み込み
    f = open(resFile);
    #*dataまでskip
    while True:
        line = f.readline()
        if line[:len("*data")] == "*data":
            break
    #node数読み込み
    line = f.readline()
    nNodes = int(line.split()[0])
    #節点の値の位置を取得
    _dummy = f.readline()
    line = f.readline()
    words = line.split()
    nums = list(map(int, words))
    nVals = sum(nums)
    dispLoc = 0
    flag = 0
    for i in range(len(words)):
        valName = f.readline().split()[0]
        if valName == "DISPLACEMENT":
            flag = 1
        else:
            if flag == 0:
                dispLoc += nums[i]
    #DISPLACEMENTの値を読み込む
    dispDict = {}
    n = 0
    while True:
        #nodeNo取得
        line = f.readline()
        nodeNo = int(line.split()[0])
        #disp = [nodeNo]
        #disp読み込み
        vals = []
        while True:
            line = f.readline()
            words = line.split()
            vals += words
            if len(vals) >= nVals:
                break
        dispWords = vals[dispLoc:dispLoc+3]
        dispVal = list(map(float, dispWords))
        dispDict[nodeNo] = dispVal
        n += 1
        if n >= nNodes:
            break
    f.close()
    return dispDict

#
#  createMappingDispFile
#-------------------------
def createMappingDispFile(dispPtc, endStep):
    """ 変位をpatchにmappingしたfileを作成する"""
    nProcs = couplingDict["nProcs"]
    np = int(nProcs)
    queue = []
    procs = []
    for n in range(np):
        q = multiprocessing.Queue()
        queue.append(q)
        procNo = str(n)
        p = multiprocessing.Process(
            target=createMappingDispFile_proc,
            args=(dispPtc, endStep, procNo, q)
            )
        p.start()
        procs.append(p)
    #全セットprocessの終了を待つ
    for n in range(np):
        a = queue[n].get()

#
#  createMappingDispFile_proc
def createMappingDispFile_proc(dispPtc, endStep, procNo, q):
    """ 指定したprocNoの変位をpatchにmappingしたfileを作成する"""
    solidCaseDir = couplingDict["solidCaseDir"]
    mesh = mapping.fistrMesh(solidCaseDir) 
    #mappingするpointの座標を取得
    mapPointFaceDict, mapPointDict = getMappedPointLocsFsi(procNo)
    #mappingTriFileがあるか？
    if isMappingTriFileFsi(procNo) == False:
        #存在しない場合、mappingTriDictを取得
        mapStat, mappingTriDict = mesh.getMappingTriLocs(dispPtc, mapPointFaceDict, mapPointDict)
        if mapStat != "OK":
            return "error in displacement mapping."
        #mappingTriFileを作成
        createMappingTriFileFsi(mappingTriDict, procNo)
    else:
        #fileを読み込む
        mappingTriDict = readMappingTriFileFsi(procNo)
    #変位をmapping
    mappingDict = mesh.setMappingDataFromTriLocs(dispPtc, mapPointFaceDict, mappingTriDict)
    #mapping結果を保存(このfileをOpenFOAM側が読み込む)
    saveMappingDisp(mappingDict, endStep, procNo)
    q.put("")

#
#  getMappedPointLocsFsi
def getMappedPointLocsFsi(procNo):
    """ 変位をmappingするpointの座標を取得する"""
    patchName = couplingDict["mappingPatch"]
    #procNo = couplingDict["procNo"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    name = patchName + "_mapPointFaceNodes_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    mapPointFaceDict = {}
    mapPointDict = {}
    for line in lines:
        words = line.split()
        if len(words) > 4:
            nodeNo = int(words[0])
            pointLoc = list(map(float, words[1:4]))
            faceNodes = list(map(int, words[4:]))
            mapPointFaceDict[nodeNo] = [pointLoc, faceNodes]
            mapPointDict[nodeNo] = pointLoc
    return mapPointFaceDict, mapPointDict

#
#  createMappingTriFileFsi
def createMappingTriFileFsi(mappingTriDict, procNo):
    """ mapping用の3点の座標をfileに保存する"""
    #procNo = couplingDict["procNo"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    name = "mappingTriFile_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    lines = []
    for key in mappingTriDict.keys():
        faceNo = key
        word = str(faceNo)
        triLocs = mappingTriDict[key]
        loc0, loc1, loc2 = triLocs
        word0 = " ".join(list(map(str, loc0)))
        word1 = " ".join(list(map(str, loc1)))
        word2 = " ".join(list(map(str, loc2)))
        line = " ".join([word, word0, word1, word2]) + "\n"
        lines.append(line)
    f = open(fileName, "w"); f.writelines(lines); f.close()

#
#  readMappingTriFileFsi
def readMappingTriFileFsi(procNo):
    """ 保存したmapping用の3点の座標を読み込む"""
    #procNo = couplingDict["procNo"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    name = "mappingTriFile_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    mappingTriDict = {}
    for line in lines:
        words = line.split()
        faceNo = int(words[0])
        key = faceNo
        loc0 = list(map(float, words[1:4]))
        loc1 = list(map(float, words[4:7]))
        loc2 = list(map(float, words[7:10]))
        mappingTriDict[key] = [loc0, loc1, loc2]
    return mappingTriDict


#
#  isMappingTriFileFsi
def isMappingTriFileFsi(procNo):
    """ 変位mapping用の3点を取得したfileが存在するかチェック"""
    #procNo = couplingDict["procNo"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    name = "mappingTriFile_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    return os.path.exists(fileName)

#
#  saveMappingDisp
def saveMappingDisp(mappingDict, endStep, procNo):
    """ mappingDispの結果を保存。このファイルをOF側が読み込む"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    patchName = couplingDict["mappingPatch"]
    timeIndex = couplingDict["timeIndex"]
    saveDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = patchName + "_mapPointFaceNodes_" + procNo
    fileName = saveDir + "/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    #保存のfilename
    name = patchName + "_disp_" + timeIndex + "_" + procNo + "tm"
    if procNo == "0":
        #print("(py) --- displacement file is creating for OpenFOAM at " + str(endStep) + "...")
        print("(py) --- displacement file is creating for OpenFOAM at " + name + "...")
    #name = patchName + "_disp_" + str(endStep) + "_" + procNo + "tm"
    dispFile = saveDir + "/" + name
    f = open(dispFile, "w")
    for line in lines:
        words = line.split()
        nodeNo = int(words[0])
        disp = mappingDict[nodeNo]
        line = " ".join(list(map(str, disp))) + "\n"
        f.write(line)
    f.close()
    return

#
#  deleteResSetRestartFileCht
#------------------------------
def deleteResSetRestartFileCht():
    """ 結果fileを削除する。
    ただし、writeInterval時は、結果fileとrestartFileを残す"""
    timeName = couplingDict["timeName"]
    beforeTime = couplingDict["beforeTime"]
    writeInterval = couplingDict["writeInterval"]
    #deltaT = couplingDict["deltaT"]
    timeIndex = couplingDict["timeIndex"]
    solidCaseDir = couplingDict["solidCaseDir"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    patchName = couplingDict["mappingPatch"]
    nSteps = couplingDict["nSteps"]
    deltaT = str(float(timeName) - float(beforeTime))
    #writeIntervalかどうか？
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data/"
    delStep, delTime = getBeforeCouplingTimeTemp()
    #delTime = timeName
    #delStep = timeIndex
    delFiles = []
    delFolders = []
    #OpenFOAM側を確認
    #if pyCoupling.isWriteInterval(timeName, writeInterval, deltaT) == False:
    if pyCoupling.isWriteInterval(delTime, writeInterval, deltaT) == False:
        #削除可能な時間
        #  heatFluxの点群ファイルを削除
        pattern = solidCaseDir + "/" + patchName + "_faceCenterHeatFlux_" + delTime + "_*.particles"
        fileNames = getAllProcsFiles(pattern, int(nProcs))                
        delFiles += fileNames
        #  温度tempを削除
        pattern = dataDir + patchName + "_temp_" + delStep + "_*tm"
        fileNames = getAllProcsFiles(pattern, int(nProcs))
        delFiles += fileNames
        #  faceHeatFluxを削除
        pattern = dataDir + patchName + "_faceHeatFlux_" + delTime + "_*tm"
        fileNames = getAllProcsFiles(pattern, int(nProcs))
        delFiles += fileNames
    else:
        #writeInterval時の場合
        #  heatFluxの点群ファイル
        pattern = solidCaseDir + "/" + patchName + "_faceCenterHeatFlux_" + delTime + "_*.particles"
        fileNames = getAllProcsFiles(pattern, int(nProcs))                
        delFiles += fileNames
        #  温度tempをrename
        pattern = dataDir + patchName + "_temp_" + delStep + "_*tm"
        fileNames = getAllProcsFiles(pattern, int(nProcs))
        delete2charaFromEndName(fileNames)
        #  faceHeatFluxをrename
        pattern = dataDir + patchName + "_faceHeatFlux_" + delTime + "_*tm"
        fileNames = getAllProcsFiles(pattern, int(nProcs))
        delete2charaFromEndName(fileNames)
    #FrontISTR側を確認
    #    温度のFrontISTRは、addTempRenameRes関数で処理するので
    #    このでは、不要。
    # if pyCoupling.isWriteInterval(timeName, writeInterval, deltaT) == False:
    #     #削除可能な時間
    #     #  resFileを削除
    #     resFile = solidCaseDir + "/FistrModel_temp.res.0." + delStep
    #     delFiles += [resFile]
    #     #  vtuFileを削除
    #     vtuFile = solidCaseDir + "/FistrModel_cht.vis_psf." + delStep.zfill(4) + ".pvtu"
    #     delFiles += [vtuFile]
    #     #  vtuFolder
    #     vtuFolder = solidCaseDir + "/FistrModel_cht.vis_psf." + delStep.zfill(4)
    #     delFolders += [vtuFolder]
    #FrontISTRのrestartFileを保存
    calcEndTime = timeName
    calcEndIdx = timeIndex
    if pyCoupling.isWriteInterval(calcEndTime, writeInterval, deltaT) == True:
        #writeInterval時の時
        #  resFileをrenameして保存
        saveRestartFileCht(calcEndTime, calcEndIdx)
    #fileFolderを削除
    for delFile in delFiles:
        if os.path.exists(delFile) == True:
            os.remove(delFile)
    for delFolder in delFolders:
        if os.path.isdir(delFolder) == True:
            shutil.rmtree(delFolder)

#
#  getAllProcsFiles
def getAllProcsFiles(pattern, np):
    """ 全procsのfile名を取得して返す"""
    names = pattern.split("*")
    files = []
    for i in range(np):
        fileName = names[0] + str(i) + names[1]
        files.append(fileName)
    return files

#
#  delete2charaFromEndName
def delete2charaFromEndName(fileNames):
    """ fileNameの最後の2文字(tm)を削除"""
    for fileName in fileNames:
        if os.path.exists(fileName):
            os.rename(fileName, fileName[:-2])

#
#  saveRestartFileCht
def saveRestartFileCht(timeName, timeIndex):
    """ FrontISTRのrestartFileを設定する"""
    if int(timeIndex) == 0:
        return
    #restartFile保存
    solidCaseDir = couplingDict["solidCaseDir"]
    orgName = solidCaseDir + "/FistrModel_cht.restart.0"
    newName = solidCaseDir + "/FistrModel_cht.restart_" + timeName + "_" + timeIndex + ".0"
    shutil.copy(orgName, newName)


#
#  waitUntilExistingFile
#-------------------------
def waitUntilExistingFile(fileName):
    """ pressFileが出来上がるまで待つ"""
    count = waitCount
    sleepTime = waitTime
    while True:
        if os.path.exists(fileName) == True:
            time.sleep(sleepTime)
            break
        count -= 1
        if count < 0:
            break
        time.sleep(sleepTime)
    return

#-----------------------
#  isDispCoupling
#-----------------------
def isDispCoupling():
    """ 変位のcouplingかどうか確認してその結果を返す"""
    timeIndex = couplingDict["timeIndex"]
    nStepsOF = couplingDict["nStepsOF"]     #変位のstep数
    timeName = couplingDict["timeName"]
    writeInterval = couplingDict["writeInterval"]
    beforeTime = couplingDict["beforeTime"]
    deltaT = str(float(timeName) - float(beforeTime))
    if int(timeIndex) % int(nStepsOF) == 0:
        return True
    if pyCoupling.isWriteInterval(timeName, writeInterval, deltaT) == True:
        return True
    return False

#------------------
#  isTempCoupling
#------------------
def isTempCoupling():
    """ 温度のcouplingかどうか確認してその結果を返す"""
    timeIndex = couplingDict["timeIndex"]
    nSteps = couplingDict["nSteps"]     #変位のstep数
    timeName = couplingDict["timeName"]
    writeInterval = couplingDict["writeInterval"]
    beforeTime = couplingDict["beforeTime"]
    deltaT = str(float(timeName) - float(beforeTime))
    if int(timeIndex) % int(nSteps) == 0:
        return True
    if pyCoupling.isWriteInterval(timeName, writeInterval, deltaT) == True:
        return True
    return False

#------------------
#  createNextDisp
#------------------
def createNextDisp():
    """ OpenFOAMが読み込む変位のfileをコピーして作成する"""
    timeIndex = couplingDict["timeIndex"]   #endStep
    nStepsOF = couplingDict["nStepsOF"]     #変位のstep数
    nSteps = couplingDict["nSteps"]         #温度のstep数
    procNo = couplingDict["procNo"]
    startStep = int(timeIndex) - int(nStepsOF)
    patchName = couplingDict["mappingPatch"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    #温度のcoupling時に作成した変位file
    #name = patchName + "_disp_" + timeIndex + "_" + procNo + "tm"
    name = patchName + "_disp_" + str(startStep) + "_" + procNo + "tm"
    oldDispFile = dataDir + "/" + name
    if os.path.exists(oldDispFile) == False:
        oldDispFile = oldDispFile[:-2]
        if os.path.exists(oldDispFile) == False:
            return
    #OpenFOAMが読み込むfileを作成
    #calcEndStep = int(timeIndex) + int(nStepsOF)
    calcEndStep = int(timeIndex)
    if calcEndStep % int(nSteps) != 0:
        name = patchName + "_disp_" + str(calcEndStep) + "_" + procNo + "tm"
        newDispFile = dataDir + "/" + name
        shutil.copy(oldDispFile, newDispFile)
    #不要なfileを削除--------------
    timeName = couplingDict["timeName"]
    beforeTime = couplingDict["beforeTime"]
    deltaT = str(float(timeName) - float(beforeTime))
    #deltaT = couplingDict["deltaT"]
    writeInterval = couplingDict["writeInterval"]
    #name = patchName + "_disp_" + timeIndex + "_" + procNo + "tm"
    name = patchName + "_disp_" + str(startStep) + "_" + procNo + "tm"
    orgFile = dataDir + "/" + name
    #  writeInterval？
    sStep, sTime = getBeforeCouplingTimePress()
    if pyCoupling.isWriteInterval(sTime, writeInterval, deltaT) == True:
        #orgFileをrenameする
        newFile = orgFile[:-2]
        if os.path.exists(orgFile) == True:
            shutil.move(orgFile, newFile)
    else:
        #削除する
        if os.path.exists(orgFile) == True:
            os.remove(orgFile)



if __name__ == "__main__":
    import gettext
    gettext.install("app") # replace with the appropriate catalog name
    _ = gettext.gettext

    currTime = sys.argv[1]      #currTime
    beforeTime = sys.argv[2]    #前回のtime
    procNo = sys.argv[3]        #procNo
    nProcs = sys.argv[4]        #nProcs
    timeIndex = sys.argv[5]     #timeIndex
    fluidCaseDir = os.getcwd()
    couplingData = pyCoupling.couplingData(fluidCaseDir)
    couplingDict = couplingData.read()
    couplingDict["fluidCaseDir"] = fluidCaseDir
    couplingDict["timeName"] = currTime
    couplingDict["beforeTime"] = beforeTime
    couplingDict["procNo"] = procNo
    couplingDict["nProcs"] = nProcs
    couplingDict["timeIndex"] = timeIndex
    waitCount = int(couplingDict["maxWaitTime"])
    #共有メモリの設定
    mmCommon = createCommonArea()
    mmCommTimePress = createCommTimePressArea()
    mmCommTimeTemp = createCommTimeTempArea()
    #現在時間が終了時間？
    if float(couplingDict["timeName"]) > float(couplingDict["endTime"]):
        exit()
    #beforeTimeが計算開始時間よりも前 or 現時点がbeforeTime？
    if (float(beforeTime) <  float(couplingDict["startTime"]) or    #restart初回
        float(beforeTime) == float(currTime)):                      #計算初回
        #共有メモリにcouplingTimeを保存して終了
        saveCouplingTimeTemp()
        saveCouplingTimePress()
        exit() 
    #温度のcoupling？
    if isTempCoupling() == True:
        #couplingTimeを共有メモリに保存
        saveCouplingTimeTemp()
        #mainへ
        stat = coupling_chtChtss()
        if stat != "":
            raise NameError(stat)
    #変位のcoupling？
    if isDispCoupling() == True:
        #couplingTimeを共有メモリに保存
        saveCouplingTimePress()
        #cht時に計算したdispFileを作成
        createNextDisp()
        

