#!/usr/bin/python3
#
#       waitToRunFrontISTR.py
#
#       runFistrのファイル有無を確認し、あればFrontISTRを起動する
#       FrontISTR起動後は、runFistrを削除する。
#
#   23/09/28    新規作成
#      10/20    waitToRunFrontIstr,runFrontIstr:交互計算に対応させる為に
#               timeName==endTimeで終了をtimeName>endTimeで終了に変更
#      10/25    waitToRunFrontIstr:時間制限で停止させる様に修正。
#

import os, sys
import time
import subprocess
import mmap
import pyCoupling

waitCount = 5000
waitTime = 0.01

#-------------------
#  createCommonArea
#-------------------
def createCommonArea():
    """ common fileを共有メモリとして設定する"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = "common"
    commonFile = dataDir + "/" + name
    f = open(commonFile, "r+b"); mmFile = mmap.mmap(f.fileno(), 0); f.close()
    return mmFile

#
#  writeCommonArea
def writeCommonArea(cont):
    """ 共有メモリ（200文字）への書き込み。"""
    global mmCommon
    length = 200
    space = "\00" * (length - len(cont))
    cont += space
    mmCommon.seek(0)
    mmCommon.write(cont.encode())
    mmCommon.flush()

#
#  readCommonArea
def readCommonArea():
    """ 共有メモリの内容を読み込む"""
    global mmCommon
    mmCommon.seek(0)
    cont = mmCommon.read().decode()
    n = cont.find("\00")
    if n >= 0:
        cont = cont[:n]
    else:
        cont = ""
    return cont

#---------------------
#  waitToRunFrontIstr
#---------------------
def waitToRunFrontIstr():
    """ file有無を確認し、FrontISTRを起動する"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    maxWaitTime = couplingDict["maxWaitTime"]
    endTime = couplingDict["endTime"]
    waitCount = int(maxWaitTime)
    #logFileをクリア
    logFile = fluidCaseDir + "/solidSolve.log"
    f = open(logFile, "w"); f.close()
    #FrontISTRを繰り返し実行
    lastFlag = 0
    count = waitCount
    while True:
        flag, cont = getRunFistr()
        if flag == True:
            calcEndTime = runFrontIstr(cont)
            #countをリセットして繰り返す
            count = waitCount
            # if calcEndTime < float(endTime):
            #     #countをリセットして繰り返す
            #     count = waitCount
            # elif lastFlag == 1:
            #     #計算終了
            #     break
            # else:
            #     lastFlag = 1
        time.sleep(waitTime)
        count -= 1
        if count < 0:
            break

#
#  getRunFistr
#--------------
def getRunFistr():
    """ 共有メモリの内容を読み込み確認する"""
    cont = readCommonArea()
    if len(cont) > 0:
        lines = cont.split("\n")
        if len(lines) >= 3:
            words = lines[0].split()
            if len(words) >= 2:
                if words[1] == "runFistr":
                    return True, cont
    return False, ""

#
#  runFrontIstr
#----------------
def runFrontIstr(cont):
    """ FrontISTRを起動する
    全ての設定は完了している為、ここでは起動のみ"""
    solidAnaComm = couplingDict["solidAnaComm"]
    solidCaseDir = couplingDict["solidCaseDir"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    nThreads = couplingDict["nThreads"]
    endTime = couplingDict["endTime"]
    deltaT = couplingDict["deltaT"]
    nStepsOF = couplingDict["nStepsOF"]
    (timeName, msg) = getFooterMessage(cont)
    #終了確認
    if float(timeName) > float(endTime):
        #終了
        return float(timeName)
    #FrontISTR起動
    logFile = fluidCaseDir + "/solidSolve.log"
    fo = open(logFile, "a")
    t = time.time()
    saveDir = os.getcwd()
    os.chdir(solidCaseDir)
    comm = solidAnaComm + " -t " + nThreads
    proc = subprocess.run(
        comm,
        shell=True,
        stdout=fo,
        stderr=fo)
    #終了した事を共有メモリに記載
    cont = "title FistrFinished\n"
    cont += "#end\n"
    writeCommonArea(cont)
    #footerを出力
    l = time.time() - t
    l = int(l * 100 + 0.5) / 100
    line = msg + " " + str(l) + " sec\n\n"
    fo.write(line)
    fo.close()
    os.chdir(saveDir)
    #計算終了時のtimeを戻す
    calcEndTime = float(timeName) + int(nStepsOF) * float(deltaT)
    return calcEndTime

#
#  getFooterMessage
def getFooterMessage(cont):
    """ contからtimeNameとmessを取得する"""
    footer = ""
    lines = cont.split("\n")
    for line in lines:
        words = line.split()
        if len(words) >= 2:
            if words[0] == "footer":
                footer = " ".join(words[1:])
            elif words[0] == "time":
                timeName = words[1]
    return (timeName, footer)



if __name__ == "__main__":
    fluidCaseDir = os.getcwd()
    couplingData = pyCoupling.couplingData(fluidCaseDir)
    couplingDict = couplingData.read()
    couplingDict["fluidCaseDir"] = fluidCaseDir
    #共有メモリの設定
    mmCommon = createCommonArea()
    waitToRunFrontIstr()
